==> fotoalbum-0.33.tar.bz2: fotoalbum-0.33/src/base/plugin.cpp <==
// -*- mode: c; mode: fold -*-

/*
 * Copyright (C) 2006-2009  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "plugin.h"

#include <fstream>
#include <iostream>
#include <cstdlib>

#include "../config.h"
#include "treeitem.h"
#include "treeview.h"
#include "../shared/lib_conversions.h"
#include "../shared/filetools.h"
#include "../shared/scandir.h"

void Plugin::addPlugins(SimpleXMLTransfer*& xmllist,
                        SimpleXMLTransfer*  pxml)
{
  if (xmllist == 0)
  {
    xmllist = pxml;
  }
  else
  {
    SimpleXMLTransfer* item  = xmllist->getChild("item");
    SimpleXMLTransfer* pitem = pxml->getChild("item");
    int                count = pitem->getChildCount();
    SimpleXMLTransfer* plugin;
    for (int p=0; p<count; p++)
    {
      plugin = pitem->getChildAt(0);
      pitem->removeChildAt(0);

      // todo: handle internationalization here, too
      {
        std::string descr;
        bool fChanged = true;
        descr = plugin->attribute("descr");
        // Prevent duplicate names
        while (fChanged)
        {
          fChanged = false;
          for (int m=0; m<item->getChildCount(); m++)
          {
            if (item->getChildAt(m)->attribute("descr").compare(descr) == 0)
            {
              descr = itoStr(item->getChildCount(), '0', 2) + " " + descr;
              fChanged = true;
            }
          }
        }
        plugin->setAttributeOverwrite("descr", descr);
      }
      
      item->addChild(plugin);
    }
    delete pxml;
  }  
}

void Plugin::addPlugins(SimpleXMLTransfer*& xmllist,
                        std::string         newfile)
{
  SimpleXMLTransfer* pxml = new SimpleXMLTransfer(newfile);
  addPlugins(xmllist, pxml);
}

std::string Plugin::getPluginList(SimpleXMLTransfer*& xml)
{
  std::string              dir;
  ScanDir*                 scan;
  std::string              filename;
  
  xml = 0;
  
  if (WIN32_VAL != 0)
  {
    SimpleXMLTransfer* pPlugins = new SimpleXMLTransfer();
    pPlugins->setName("plugins");
    pPlugins->setAttribute("item.plugin.descr",     "Open");
    pPlugins->setAttribute("item.plugin.path",      "");
    pPlugins->setAttribute("item.plugin.args",      "\"%F\"");
    pPlugins->setAttribute("item.plugin.blocking",  "0");
    pPlugins->setAttribute("item.plugin.list",      "0");
    addPlugins(xml, pPlugins);
  }
    
  if (WIN32_VAL == 0 && system("which xdg-open") == 0)
  {
    SimpleXMLTransfer* pPlugins = new SimpleXMLTransfer();
    pPlugins->setName("plugins");
    pPlugins->setAttribute("item.plugin.descr",     "Open");
    pPlugins->setAttribute("item.plugin.path",      "xdg-open");
    pPlugins->setAttribute("item.plugin.args",      "\"%F\"");
    pPlugins->setAttribute("item.plugin.blocking",  "0");
    pPlugins->setAttribute("item.plugin.list",      "0");
    addPlugins(xml, pPlugins);
  }
    
  // load old style single file for compatibility
  filename = FileTools::getHomeDir("Fotoalbum") + "plugins.xml";
  if (FileTools::fileExists(filename))
  {
    try
    {
      addPlugins(xml, filename);
    }
    catch (XMLException e)
    {
      return("Error in plugin file " + filename + ": " + e.what());
    }  
  }
  
  // 
  for (int n=0; n<2; n++)
  {
    switch (n)
    {
      case 1:
#ifdef WIN32
        // win32, relative to current dir
        dir = "../share/fotoalbum/plugins/";
#else
        dir = STR_INSTALL_PREFIX "/share/fotoalbum/plugins/";
#endif
        break;
        
      default:
        dir = FileTools::getHomeDir("Fotoalbum") + "plugins/";
        break;
    }
    scan = new ScanDir(dir.c_str());
    for (int n=0; n<scan->nNrEntries; n++)
    {
      filename = dir + (scan->list)[n]->d_name;
      if (!FileTools::isDir(filename) && filename.find(".xml") == filename.length()-4)
      {
        try
        {
          addPlugins(xml, filename);          
        }
        catch (XMLException e)
        {
          return("Error in plugin file " + filename + ": " + e.what());
        }
      }
    }
    delete scan;
  }
  
  return("");
}

void Plugin::loadLists(SimpleXMLTransfer*   plugin,
                       SimpleXMLTransfer*&  blacklist,
                       SimpleXMLTransfer*&  whitelist) 
{
  SimpleXMLTransfer* i;
  int index;

  blacklist = (SimpleXMLTransfer*)0;
  whitelist = (SimpleXMLTransfer*)0;

  index = plugin->indexOfChild("suffix");
  if (index >= 0)
  {
    i = plugin->getChildAt(index);

    index = i->indexOfChild("blacklist");
    if (index >= 0)
      blacklist = i->getChildAt(index);

    index = i->indexOfChild("whitelist");
    if (index >= 0)
      whitelist = i->getChildAt(index);
  }
}


bool Plugin::listMatch(SimpleXMLTransfer* list,
                       std::string        name) 
{
  unsigned int       size = list->getChildCount();
  SimpleXMLTransfer* i;
  std::string        suff;
  
  for (unsigned int cnt=0; cnt<size; cnt++)
  {
    i = list->getChildAt(cnt);
    suff = strU(i->attribute("str"));    
    if (name.find(suff) == name.length()-suff.length())
      return(true);
  }
  return(false);
}


bool Plugin::match(std::vector<TreeItem*> list,
                   SimpleXMLTransfer*     plugin) 
{
  SimpleXMLTransfer* bl;
  SimpleXMLTransfer* wl;
  bool               fRun;
  unsigned int       size;
  std::string        uname;
  
  loadLists(plugin, bl, wl);
  
  size = list.size();
  
  for (unsigned int cnt=0; cnt<size; cnt++)
  {
    uname = strU(FileTools::name( list[cnt]->getText() ));
        
    if (wl == (SimpleXMLTransfer*)0)
      fRun = true;
    else
      fRun = listMatch(wl, uname);
    
    if (bl != (SimpleXMLTransfer*)0 && listMatch(bl, uname))
      fRun = false;
    
    if (fRun)
      return(true);
  }
  
  return(false);
}



std::string Plugin::run(std::vector<TreeItem*> list,
                        SimpleXMLTransfer*     plugin,
                        TreeView*              data) 
{
  std::list<std::string> vCmds;
  std::string strErr = Plugin::getCmds(list, plugin, data, vCmds);
  if (strErr.empty())
    for (std::list<std::string>::iterator it = vCmds.begin(); it!=vCmds.end(); it++)
    {
      std::cout << "Executing " << *it << "\n";
      if (system((*it).c_str()))
        return("Error? Plugin returned non-zero.");
    }

  return(strErr);
}

std::string Plugin::getCmds(std::vector<TreeItem*>  list,
                            SimpleXMLTransfer*      plugin,
                            TreeView*               data,
                            std::list<std::string>& vCmds)
{
  unsigned int size  = list.size();
  std::string  ppath = trim(plugin->attribute("path"));

  if (!ppath.empty() || WIN32_VAL != 0)
  {
    std::string        fpath;
    std::string        uname;
    std::string        argin;
    std::string        argout;
    unsigned int       ailen;
    TreePicBase*       item;
    bool               fBlock;
    SimpleXMLTransfer* bl;
    SimpleXMLTransfer* wl;
    bool               fRun;
    bool               fList;

    loadLists(plugin, bl, wl);
    
    if (!ppath.empty())
      ppath = FileTools::cleanString(ppath, "/-.") + " ";
    argin = trim(plugin->attribute("args"));
    ailen = argin.length();
    fBlock= (plugin->attributeAsInt("blocking") == 1);
    fList = (plugin->attributeAsInt("list") == 1);

    if (fList)
      argout = ppath;
    
    for (unsigned int cnt=0; cnt<size; cnt++)
    {
      item  = (TreePicBase*)list[cnt];
      fpath = data->getPicPath(item);

      uname = strU(FileTools::name(fpath));

      if (wl == (SimpleXMLTransfer*)0)
        fRun = true;
      else
        fRun = listMatch(wl, uname);

      if (bl != (SimpleXMLTransfer*)0 && listMatch(bl, uname))
        fRun = false;

      if (fRun) 
      {
        if (!fList)
          argout = ppath;
        
        for (unsigned int cnt2=0; cnt2<ailen; cnt2++)
        {
          if (argin[cnt2] == '%')
          {
            if (cnt2 < ailen-1)
            {
              cnt2++;

              if (argin[cnt2] == 'f')
              {
                argout += FileTools::name(fpath);
              }
              else if (argin[cnt2] == 'F')
              {
                argout += fpath;
              }
              else if (argin[cnt2] == 'd')
              {
                argout += itoStr(item->getDateTime().getDay(), '0', 2);
              }
              else if (argin[cnt2] == 'm')
              {
                argout += itoStr(item->getDateTime().getMonth(), '0', 2);
              }
              else if (argin[cnt2] == 'Y')
              {
                argout += itoStr(item->getDateTime().getYear(), '0', 4);
              }
              else if (argin[cnt2] == 'H')
              {
                argout += itoStr(item->getDateTime().getHour(), '0', 2);
              }
              else if (argin[cnt2] == 'M')
              {
                argout += itoStr(item->getDateTime().getMinute(), '0', 2);
              }
              else if (argin[cnt2] == 'c')
              {
                argout += item->getComment();
              }
              else if (argin[cnt2] == 'k')
              {
                argout += item->getKeywords();
              }
              else if (argin[cnt2] == '%')
              {
                argout += "%";
              }
              else
                return("Error in args: unknown %" + argin[cnt2]);
            }
            else
            {
              return("Error in args: character expected after %");
            }
          }
          else
            argout += argin[cnt2];
        }
        
        if (fList)
          argout += " ";
      }
            
      if (!fList)
      {
        if (!fBlock)
          argout += " &";      
        vCmds.push_back(argout);
      }

    }
    
    if (fList)
    {
      if (!fBlock)
        argout += " &";      
      vCmds.push_back(argout);
    }
  }
  else
  {
    return("Plugin path is empty");
  }

  return("");
}

==> fotoalbum-0.104.tar.bz2: fotoalbum-0.104/src/base/plugin.cpp <==
// -*- mode: c; mode: fold -*-

/*
 * Copyright (C) 2006-2009  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "plugin.h"

#include <fstream>
#include <iostream>
#include <cstdlib>

#include "../config.h"
#include "treeitem.h"
#include "treeview.h"
#include "../shared/lib_conversions.h"
#include "../shared/filetools.h"
#include "../shared/scandir.h"
#include "../shared/platform.h"
#include "../shared/utf8.h"

void Plugin::addPlugins(SimpleXMLTransfer*& xmllist,
                        SimpleXMLTransfer*  pxml)
{
  if (xmllist == 0)
  {
    xmllist = pxml;
  }
  else
  {
    SimpleXMLTransfer* item  = xmllist->getChild("item");
    SimpleXMLTransfer* pitem = pxml->getChild("item");
    int                count = pitem->getChildCount();
    SimpleXMLTransfer* plugin;
    for (int p=0; p<count; p++)
    {
      plugin = pitem->getChildAt(0);
      pitem->removeChildAt(0);

      // todo: handle internationalization here, too
      {
        std::string descr;
        bool fChanged = true;
        descr = plugin->attribute("descr");
        // Prevent duplicate names
        while (fChanged)
        {
          fChanged = false;
          for (int m=0; m<item->getChildCount(); m++)
          {
            if (item->getChildAt(m)->attribute("descr").compare(descr) == 0)
            {
              descr = itoStr(item->getChildCount(), '0', 2) + " " + descr;
              fChanged = true;
            }
          }
        }
        plugin->setAttributeOverwrite("descr", descr);
      }
      
      item->addChild(plugin);
    }
    delete pxml;
  }  
}

void Plugin::addPlugins(SimpleXMLTransfer*& xmllist,
                        std::string         newfile)
{
  SimpleXMLTransfer* pxml = new SimpleXMLTransfer(newfile);
  addPlugins(xmllist, pxml);
}

std::string Plugin::getPluginList(SimpleXMLTransfer*& xml)
{
  std::string              dir;
  ScanDir*                 scan;
  std::string              filename;
  
  xml = 0;
  
  if (WIN32_VAL != 0)
  {
    SimpleXMLTransfer* pPlugins = new SimpleXMLTransfer();
    pPlugins->setName("plugins");
    pPlugins->setAttribute("item.plugin.descr",     "Open");
    pPlugins->setAttribute("item.plugin.path",      "");
    pPlugins->setAttribute("item.plugin.args",      "\"%F\"");
    pPlugins->setAttribute("item.plugin.blocking",  "0");
    pPlugins->setAttribute("item.plugin.list",      "0");
    addPlugins(xml, pPlugins);
  }
    
  if (WIN32_VAL == 0 && system("which xdg-open") == 0)
  {
    SimpleXMLTransfer* pPlugins = new SimpleXMLTransfer();
    pPlugins->setName("plugins");
    pPlugins->setAttribute("item.plugin.descr",     "Open");
    pPlugins->setAttribute("item.plugin.path",      "xdg-open");
    pPlugins->setAttribute("item.plugin.args",      "\"%F\"");
    pPlugins->setAttribute("item.plugin.blocking",  "0");
    pPlugins->setAttribute("item.plugin.list",      "0");
    addPlugins(xml, pPlugins);
  }
    
  // load old style single file for compatibility
  filename = FileTools::getHomeDir("Fotoalbum") + "plugins.xml";
  if (FileTools::fileExists(filename))
  {
    try
    {
      addPlugins(xml, filename);
    }
    catch (XMLException e)
    {
      return("Error in plugin file " + filename + ": " + e.what());
    }  
  }
  
  // 
  for (int n=0; n<2; n++)
  {
    switch (n)
    {
      case 1:
#ifdef WIN32
        // win32, relative to current dir
        dir = "../share/fotoalbum/plugins/";
#else
        dir = STR_INSTALL_PREFIX "/share/fotoalbum/plugins/";
#endif
        break;
        
      default:
        dir = FileTools::getHomeDir("Fotoalbum") + "plugins/";
        break;
    }
    scan = new ScanDir(dir.c_str());
    for (int n=0; n<scan->nNrEntries; n++)
    {
      filename = dir + (scan->list)[n]->d_name;
      if (!FileTools::isDir(filename) && filename.find(".xml") == filename.length()-4)
      {
        try
        {
          addPlugins(xml, filename);          
        }
        catch (XMLException e)
        {
          return("Error in plugin file " + filename + ": " + e.what());
        }
      }
    }
    delete scan;
  }
  
  return("");
}

void Plugin::loadLists(SimpleXMLTransfer*   plugin,
                       SimpleXMLTransfer*&  blacklist,
                       SimpleXMLTransfer*&  whitelist) 
{
  SimpleXMLTransfer* i;
  int index;

  blacklist = (SimpleXMLTransfer*)0;
  whitelist = (SimpleXMLTransfer*)0;

  index = plugin->indexOfChild("suffix");
  if (index >= 0)
  {
    i = plugin->getChildAt(index);

    index = i->indexOfChild("blacklist");
    if (index >= 0)
      blacklist = i->getChildAt(index);

    index = i->indexOfChild("whitelist");
    if (index >= 0)
      whitelist = i->getChildAt(index);
  }
}


bool Plugin::listMatch(SimpleXMLTransfer* list,
                       std::string        name) 
{
  unsigned int       size = list->getChildCount();
  SimpleXMLTransfer* i;
  std::string        suff;
  
  for (unsigned int cnt=0; cnt<size; cnt++)
  {
    i = list->getChildAt(cnt);
    suff = strU(i->attribute("str"));    
    if (name.find(suff) == name.length()-suff.length())
      return(true);
  }
  return(false);
}


bool Plugin::match(std::vector<TreeItem*> list,
                   SimpleXMLTransfer*     plugin) 
{
  SimpleXMLTransfer* bl;
  SimpleXMLTransfer* wl;
  bool               fRun;
  unsigned int       size;
  std::string        uname;
  
  loadLists(plugin, bl, wl);
  
  size = list.size();
  
  for (unsigned int cnt=0; cnt<size; cnt++)
  {
    uname = strU(FileTools::name( list[cnt]->getText() ));
        
    if (wl == (SimpleXMLTransfer*)0)
      fRun = true;
    else
      fRun = listMatch(wl, uname);
    
    if (bl != (SimpleXMLTransfer*)0 && listMatch(bl, uname))
      fRun = false;
    
    if (fRun)
      return(true);
  }
  
  return(false);
}



std::string Plugin::run(std::vector<TreeItem*> list,
                        SimpleXMLTransfer*     plugin,
                        TreeView*              data) 
{
  std::list<std::string> vCmds;
  std::string strErr = Plugin::getCmds(list, plugin, data, vCmds);
  if (strErr.empty())
    for (std::list<std::string>::iterator it = vCmds.begin(); it!=vCmds.end(); it++)
    {
      std::cout << "Executing " << *it << "\n";
      if (Platform::system((*it)))
        return("Error? Plugin returned non-zero.");
    }

  return(strErr);
}

std::string Plugin::getCmds(std::vector<TreeItem*>  list,
                            SimpleXMLTransfer*      plugin,
                            TreeView*               data,
                            std::list<std::string>& vCmds)
{
  unsigned int size  = list.size();
  std::string  ppath = UTF8::trim(plugin->attribute("path"));

  if (!ppath.empty() || WIN32_VAL != 0)
  {
    std::string        fpath;
    std::string        uname;
    std::string        argin;
    std::string        argout;
    unsigned int       ailen;
    TreePicBase*       item;
    bool               fBlock;
    SimpleXMLTransfer* bl;
    SimpleXMLTransfer* wl;
    bool               fRun;
    bool               fList;

    loadLists(plugin, bl, wl);

    if (!ppath.empty())
    {
      // ppath = FileTools::cleanString(ppath, "/-.") + " "; don't know why this should be used
      ppath.push_back(' ');
    }
    argin = UTF8::trim(plugin->attribute("args"));
    ailen = argin.length();
    fBlock= (plugin->attributeAsInt("blocking") == 1);
    fList = (plugin->attributeAsInt("list") == 1);

    if (fList)
      argout = ppath;
    
    for (unsigned int cnt=0; cnt<size; cnt++)
    {
      item  = (TreePicBase*)list[cnt];
      fpath = data->getPicPath(item);

      uname = strU(FileTools::name(fpath));

      if (wl == (SimpleXMLTransfer*)0)
        fRun = true;
      else
        fRun = listMatch(wl, uname);

      if (bl != (SimpleXMLTransfer*)0 && listMatch(bl, uname))
        fRun = false;

      if (fRun) 
      {
        if (!fList)
          argout = ppath;
        
        for (unsigned int cnt2=0; cnt2<ailen; cnt2++)
        {
          if (argin[cnt2] == '%')
          {
            if (cnt2 < ailen-1)
            {
              cnt2++;

              if (argin[cnt2] == 'f')
              {
                argout += FileTools::name(fpath);
              }
              else if (argin[cnt2] == 'F')
              {
                argout += fpath;
              }
              else if (argin[cnt2] == 'd')
              {
                argout += itoStr(item->getDateTime().getDay(), '0', 2);
              }
              else if (argin[cnt2] == 'm')
              {
                argout += itoStr(item->getDateTime().getMonth(), '0', 2);
              }
              else if (argin[cnt2] == 'Y')
              {
                argout += itoStr(item->getDateTime().getYear(), '0', 4);
              }
              else if (argin[cnt2] == 'H')
              {
                argout += itoStr(item->getDateTime().getHour(), '0', 2);
              }
              else if (argin[cnt2] == 'M')
              {
                argout += itoStr(item->getDateTime().getMinute(), '0', 2);
              }
              else if (argin[cnt2] == 'c')
              {
                argout += item->getComment();
              }
              else if (argin[cnt2] == 'k')
              {
                argout += item->getKeywords();
              }
              else if (argin[cnt2] == '%')
              {
                argout += "%";
              }
              else
                return("Error in args: unknown %" + argin[cnt2]);
            }
            else
            {
              return("Error in args: character expected after %");
            }
          }
          else
            argout += argin[cnt2];
        }
        
        if (fList)
          argout += " ";
      }
            
      if (!fList)
      {
        if (!fBlock)
          argout += " &";      
        vCmds.push_back(argout);
      }

    }
    
    if (fList)
    {
      if (!fBlock)
        argout += " &";      
      vCmds.push_back(argout);
    }
  }
  else
  {
    return("Plugin path is empty");
  }

  return("");
}
